@@ -39,6 +39,10 @@
 #include <io.h>
 #include <windows.h>
 
+#ifndef EOPNOTSUPP
+# define EOPNOTSUPP ENOSYS
+#endif
+
 /* Enable tracing.  The value is the module name to be printed.  */
 /*#define ENABLE_TRACING "estream" */
 
@@ -164,8 +168,6 @@ reader (void *arg)
       if (nread < 0)
         {
           ctx->error_code = (int) errno;
-          /* NOTE (W32CE): Do not ignore ERROR_BUSY!  Check at
-             least stop_me if that happens.  */
           if (ctx->error_code == ERROR_BROKEN_PIPE)
             {
               ctx->eof = 1;
@@ -264,12 +266,7 @@ create_reader (estream_cookie_w32_pollable_t pcookie)
 
   InitializeCriticalSection (&ctx->mutex);
 
-#ifdef HAVE_W32CE_SYSTEM
-  ctx->thread_hd = CreateThread (&sec_attr, 64 * 1024, reader, ctx,
-				 STACK_SIZE_PARAM_IS_A_RESERVATION, &tid);
-#else
   ctx->thread_hd = CreateThread (&sec_attr, 0, reader, ctx, 0, &tid);
-#endif
 
   if (!ctx->thread_hd)
     {
@@ -310,22 +307,6 @@ destroy_reader (struct reader_context_s *ctx)
     SetEvent (ctx->have_space_ev);
   LeaveCriticalSection (&ctx->mutex);
 
-#ifdef HAVE_W32CE_SYSTEM
-  /* Scenario: We never create a full pipe, but already started
-     reading.  Then we need to unblock the reader in the pipe driver
-     to make our reader thread notice that we want it to go away.  */
-
-  if (ctx->file_hd != INVALID_HANDLE_VALUE)
-    {
-      if (!DeviceIoControl (ctx->file_hd, GPGCEDEV_IOCTL_UNBLOCK,
-			NULL, 0, NULL, 0, NULL, NULL))
-	{
-	  trace (("%p: unblock control call failed: ec=%d",
-                  ctx, (int)GetLastError ()));
-	}
-    }
-#endif
-
   /* XXX is it feasible to unblock the thread?  */
 
   /* After setting this event CTX is void. */
@@ -567,12 +548,7 @@ create_writer (estream_cookie_w32_pollable_t pcookie)
 
   InitializeCriticalSection (&ctx->mutex);
 
-#ifdef HAVE_W32CE_SYSTEM
-  ctx->thread_hd = CreateThread (&sec_attr, 64 * 1024, writer, ctx,
-				 STACK_SIZE_PARAM_IS_A_RESERVATION, &tid);
-#else
   ctx->thread_hd = CreateThread (&sec_attr, 0, writer, ctx, 0, &tid );
-#endif
 
   if (!ctx->thread_hd)
     {
@@ -617,20 +593,6 @@ destroy_writer (struct writer_context_s *ctx)
   /* Give the writer a chance to flush the buffer.  */
   WaitForSingleObject (ctx->is_empty, INFINITE);
 
-#ifdef HAVE_W32CE_SYSTEM
-  /* Scenario: We never create a full pipe, but already started
-     writing more than the pipe buffer.  Then we need to unblock the
-     writer in the pipe driver to make our writer thread notice that
-     we want it to go away.  */
-
-  if (!DeviceIoControl (ctx->file_hd, GPGCEDEV_IOCTL_UNBLOCK,
-			NULL, 0, NULL, 0, NULL, NULL))
-    {
-      trace (("%p: unblock control call failed: ec=%d",
-              ctx, (int)GetLastError ()));
-    }
-#endif
-
   /* After setting this event CTX is void.  */
   trace (("%p: set close_ev", ctx));
   SetEvent (ctx->close_ev);
@@ -861,11 +823,25 @@ _gpgrt_w32_poll (gpgrt_poll_t *fds, size_t nfds, int timeout)
     trace_append (("%d/%c ", waitidx[i], waitinfo[i]));
   trace_finish (("]"));
 #endif /*ENABLE_TRACING*/
-  if (!any)
-    return 0;
 
-  code = WaitForMultipleObjects (nwait, waitbuf, 0,
-                                 timeout == -1 ? INFINITE : timeout);
+  if (!any)
+    {
+      /* WFMO needs at least one object, thus we use use sleep here.
+       * INFINITE wait does not make any sense in this case, so we
+       * error out. */
+      if (timeout == -1)
+        {
+          _gpg_err_set_errno (EINVAL);
+          return -1;
+        }
+      if (timeout)
+        Sleep (timeout);
+      code = WAIT_TIMEOUT;
+    }
+  else
+    code = WaitForMultipleObjects (nwait, waitbuf, 0,
+                                   timeout == -1 ? INFINITE : timeout);
+
   if (code < WAIT_OBJECT_0 + nwait)
     {
       /* This WFMO is a really silly function: It does return either
